
	
/*******************************************************************************
* Create final annual datasets, combining info from all sources, and constructing 
* households 
*
* Part A: Console for running  makedata_* files
* Part B: Create big files (individual-level)
* Part C: Create household ID
* Part D: household collapse and saves big files household-level
* Part E: Add disclosed wealth
* Part F: Creates small files
*  
*******************************************************************************/

clear
set more off

	global prog   "\\...\CLEAN\prog"
	global source "\\...\source_data\no"
	global data   "\\...\CLEAN\data"
    global temp   "\\...\data\no-backup"

************************************************************************
*** PART A: BUILD INCOME, WEALTH, ETC., DATASETS ***
************************************************************************

* Create voluntary disclosure variables
    do " $prog\makedata0_disclosersCLEAN.do"	

* Create socio-demograpchics data "$data\perskar_`yr'.dta
	do "$prog\makedata1_individual_caracteristicsCLEAN.do"

* Create income data	
	do "$prog\makedata2_wealthCLEAN.do"

* Create wealth data
	do "$prog\makedata2_incomeCLEAN.do"
	
* Create data with firm level variables
	do "$prog\makedata3_firm_infoCLEAN.do"

* Create ultimate ownership data, individual level ownership shares (firm level also exist)
	do "$prog\makedata4_ownershipCLEAN.do"

* Create a data set with dummies for firm level avoidance behavior (dividend maximization in 2005 or holding company)
 	do "$prog\makedata6_indlevelfirminfoCLEAN.do"

* Allocate operating profits of non-VPS firms to individual shareholder  --> $data\operating_nonvps_`yr'	
    do "$prog\makedata7_profits_ownersCLEAN.do"	


************************************************************************
*** PART B: CREATES BIG FILES BY COMBINING DATASETS ***
************************************************************************

cap log close
log using $prog\do_smallfilesCLEAN_sd.log, replace

foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {

di "RUNNING YEAR 20`yr'"

* Start from socio-economic characteristics
use $data\perskar_`yr'.dta, clear
	de, fullname 
	di "NUMBER OF INDIVIDUALS IN PERSKAR IN YEAR 20`yr' = `r(N)'"

* Make dummy for receiving wealth tax reduction under the 80% rule current year
	qui replace avoid_80reduction_amount = 0 if avoid_80reduction_amount == .
	qui gen avoid_80reduction = (avoid_80reduction_amount > 0)
	label var avoid_80reduction "tax reduction 80perc rule current year"

* Add wealth data
	merge 1:1 ID using $data\WEALTH_`yr'.dta, keep(match master) nogen
	qui su mwealth if mwealth == ., meanonly
	di "NUMBER OF INDIVIDUALS WITH NO WEALTH DATA IN YEAR 20`yr' = `r(N)'"
	local tot_wealth_`yr' = round(r(sum) / 1e9, .1)
	di "TOTAL WEALTH AFTER MERGING WITH PERSKAR IN YEAR 20`yr' = tot_wealth_`yr' BN NOK"

* Create dummy for having claimed foreign assets or foreign deposits
    cap drop year
	gen year=20`yr'
	
	if year < 2009 gen a_pf_business = . // foreign business assets doen't exist before 2009
	foreach var in a_pf_housing  a_pf_business a_pf_shares a_pf_bonds a_pf_deposits { 
		qui gen x`var' = (`var' != . & `var' != 0)
	}
	qui gen has_reported_foreign_assets = (xa_pf_housing + xa_pf_business + xa_pf_shares + xa_pf_bonds + xa_pf_deposits > 0)
	label var has_reported_foreign_assets "Has claimed (paastand) foreign assets"	
	tab has_reported_foreign_assets, m
	qui gen has_reported_foreign_deposits = (xa_pf_deposits > 0)
	label var has_reported_foreign_deposits "Has claimed (paastand) foreign deposits" 	
	tab has_reported_foreign_deposits, m
 	drop xa_pf* year
	 
* Add income data
	merge 1:1 ID using $data\income_`yr'.dta, keep(match master) nogen 

* Create dummies for residency status
	qui gen ind_status_1=(ind_status==1)
	qui gen ind_status_3=(ind_status==3)
	qui gen ind_status_5=(ind_status==5)
	qui gen ind_status_miss=(ind_status==.)
	label var ind_status_1 "ind. registered as resident"
	label var ind_status_3 "ind. registered as emigrated"
	label var ind_status_5 "ind. registered as dead"
	label var ind_status_miss "ind. register status missing"
    cap drop year
	
* Add info on whether majority-owned is holding companpany, dividend maximizer
	merge 1:1 ID using $data\indlevelfirminfo_avoid.dta, keep(match master) nogen 
	d avoid_*, fullname
	sum avoid_* 
    cap drop year
	
* Add annual dummies for holding companies current year
	merge 1:1 ID using $data\annualholding_`yr'.dta, keep(match master) nogen
	d avoid_*, fullname
	sum avoid_* 
	cap drop year
	
* Add info from personal level pip/BKP crackdowns at any point
	merge 1:1 ID using $source\PIP_ind_CLEANED.dta, keep(match master) nogen
	d aud_*, fullname
	sum aud_*
	cap drop year
	
* add info on a parent being discloser
	cap drop parent_year_of_contact
	cap drop parent_evader
	
	merge 1:1 ID using $data\parent_vd.dta
	drop if _m==2
	drop _m
	sort ID

* Replacing missing by zeros for avoidance dummies
	foreach var in parent_evader avoid_80reduction_amount aud_BKP aud_insample avoid_divmax05_prof_infirm avoid_holding avoid_divmax05_prof avoid_holding_establ_05 avoid_holding_establ_pre09 avoid_holding2 avoid_holding2_establ_05 avoid_holding2_establ_pre09 avoid_holding2_establ_post09 avoid_holding_currentown avoid_holding_currentfound avoid_holding2_currentown avoid_holding2_currentfound {
		qui replace `var'=0 if  `var'==.
	}

* make year variable now in the final sample
	 cap drop year
	 gen year=20`yr'
	
* Create dummies for immigration, emigration, and emigration destinations  
	cap drop ind_emigrates_thisyear 
	cap drop ind_immigrates_thisyear
	qui gen ind_emigrates_thisyear=(ind_e_country!="")
	label var ind_emigrates_thisyear "emigrates during the year"
	qui gen ind_immigrates_thisyear=(ind_i_country!="")
	label var ind_immigrates_thisyear "immigrates during the year"
	qui gen ind_emigrates_haven=0
	local havens "BE AD GI MT LI LU MC SM CH AT LR SC BH HK CY MO MY SG VI BB AG BZ BS BM VG KY CR DM GD MS AW SX BQ NI KN LC VC TT TC CL UY AS CK VU NR NU MH"
	foreach haven of local havens {
		qui replace ind_emigrates_haven=1 if ind_e_country=="`haven'" & ind_emigrates_thisyear==1
	}
	label var ind_emigrates_haven "emigrates to tax haven during the year year"
	qui gen ind_emigrates_nohaven=0
	qui replace ind_emigrates_nohaven=1 if ind_emigrates_thisyear==1 & ind_emigrates_haven==0
	label var ind_emigrates_nohaven "emigrates to non-tax haven during the year"

* Create dummies for foreign origin (either foreign citizen or Norwegian citizen born abroad)
	qui gen ind_foreign=(ind_foreign_country!="NO")
	label var ind_foreign "dummy foreign nationality or birth country"
	
* Conversion of remaining string variables into numeric and drop strings
	destring ind_county, replace
	destring ind_edulevel, replace
	cap drop ind_foreign_country
	cap drop ind_e_country
	cap drop ind_i_country 
	cap drop blurring 
	cap drop tax_all_blurred
	cap drop tax_income_blurred
	cap drop tax_wealth_blurred
	cap drop ind_foreigner 
	cap drop vd_disclosed_country
    
	replace ind_ID_spouse=. if ind_ID_spouse==0
	compress
	sort ID
save $temp\big`yr'.dta, replace
} 
 

*************************************************************************************
* PART C: ADD HOUSEHOLD ID *
*************************************************************************************

* Make parent data set
	use $source\slekt.dta, clear
	keep if KNY_KNYTNING_TYPE=="FORE"
	drop KNY_KNYTNING_TYPE
	rename FNR_FRA_NN ID
	rename FNR_TIL_NN ind_par
	sort ind_par
	save $temp\par.dta, replace

* Define individuals with income but missing wealth of low age as children
foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {
	use $temp\big`yr'.dta, clear
	keep ID ind_age ind_ID_spouse ind_marital ind_status ind_male taxwealth i_inckg 
	sort ID 
	save $temp\spouse_`yr'.dta, replace
	keep if (ind_age<20 & ind_ID_spouse==.) | (ind_age<26 & i_inckg!=. & taxwealth==.)
	keep ID
	sort ID
	gen child=1
	merge 1:m ID using $temp\par.dta
	drop if _m==2
	drop _m
	rename ID id_child
	rename ind_par ID
	sort ID
	save $temp\parents_`yr'.dta, replace
}

* Make household indicator: common id for both spouses if married (not if separated) and kids.
* Follow convention in the literature and use the man's ID as a household ID for the couple.

 foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {
	use $temp\spouse_`yr', clear
	 gen ind_hhold=ID
	 
	 replace ind_hhold=ind_ID_spouse if ind_male==0 & ind_marital==2 & ind_ID_spouse!=. 
	 sort ID
	 
	save $temp\temp1, replace
	 drop if (ind_age<20 & ind_ID_spouse==.) | (ind_age<26 & i_inckg!=. & taxwealth==.)
	 sort ID
	save $temp\temp2, replace

	use $temp\parents_`yr'.dta, clear
	 merge n:1 ID using $temp\temp2
	 drop if _m==2
	 drop _m
	 replace ind_hhold=id_child if ind_hhold==.
     *children might have two different household numbers if the parents are divorced
     *thus, attribute them the household number of the mother as the first choice, if duplicates exist
	 sort id_child ind_male
	 by id_child: keep if _n==1
	 keep id_child ind_hhold
	 rename id_child ID

	 append using $temp\temp2
	 duplicates report ID
	 keep ID ind_hhold
	 drop if ID==.
	 label variable ind_hhold "householdID. If married use man/father's id, if parents divorced, child has mother's family ID"
	 
	 gen head_hhold=0
	 replace head_hhold=1 if ID==ind_hhold
	 label variable head_hhold "male head of household"
	 compress
	 sort ID
	save $temp\hhold_`yr'.dta, replace
	
	* prepare spousal characteristics to keep in the main data set.
	use $temp\spouse_`yr', clear
	keep ID ind_age ind_status taxwealth i_inckg 
	rename ID ind_ID_spouse 
	rename ind_age spouse_age
	rename ind_status spouse_status
	rename taxwealth spouse_taxwealth
	rename i_inckg spouse_inckg 
	sort ind_ID_spouse -spouse_age
	by ind_ID_spouse: keep if _n==1
	save $temp\spouseinfo_`yr', replace
	}

* Add household ID and spouse info to big files 

* for 2001 we do not have personal characteristics, so status, spouse etc info are then 
* same as for 2002
 foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {  
	
	use $temp\big`yr'.dta, clear
	 cap drop ind_hhold
	 sort ind_ID_spouse

	 merge n:1 ind_ID_spouse using $temp\spouseinfo_`yr'
	 drop if _m==2
	 drop _m
	 
	 sort ID
	 merge 1:1 ID using $temp\hhold_`yr'.dta
	 drop if _m==2
	 drop _m
	 replace ind_hhold=ID if ind_hhold==.
	 d, fullname
	 sum
	 compress
	 save $data\big`yr'.dta, replace
}

*************************************************************************************
*** PART D: HOUSEHOLD COLLAPSE ***
*************************************************************************************

foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {  

use $data\big`yr'.dta, clear

di "Drop households for which all members have missing mwealth info [dead, etc.]"
	 qui gen nowealthdata = !mi(mwealth)
	 bysort ind_hhold: egen nowealthdatahhold=total(nowealthdata)
	 drop if nowealthdatahhold==0
	 qui drop nowealthdatahhold nowealthdata

* Save labels
	foreach v of var _all {
		local l`v' : variable label `v'
		if `"`l`v''"' == "" local l`v' "`v'"
	}

* Collapse at household level. Keep induvidual characteristics of head of household. 
	// (sum)  treats missing as 0; 
	// (mean) treats missing as missing
	// (min)  returns missing if and only if all values are missing	

	 gsort ind_hhold -head_hhold -ind_age
	 #delimit;
	 collapse 
	 (first) year ID ind_ID_spouse ind_status ind_marital ind_male ind_age ind_children ind_county spouse_age spouse_status spouse_taxwealth spouse_inckg /* keep socio demographic charac of eldest hhold member for these variables and of spouse */
	 (min)   vd_year_of_contact vd_year_of_decision  aud_year_start aud_year_decision 
	 (sum)   a_* d_* p_* tax* da_* m* vd_taxed_* i_* di_*  avoid_80reduction_amount /* sums for all quantitative variables */
	 (max) 	ind_divorced ind_married  ind_edulevel ind_university ind_edu_bus ind_foreign ind_emigrates* ind_immigrates*  
	 			vd_evader vd_case* parent_evader
	 			has_reported_foreign_assets has_reported_foreign_deposits 
	 			aud_BKP aud_insample 
	 			avoid_divmax05_prof_infirm avoid_holding avoid_holding_establ_05 avoid_holding_establ_pre09  
				avoid_80reduction_ever avoid_80reduction avoid_holding2 avoid_holding2_establ_05 avoid_holding2_establ_pre09 avoid_holding2_establ_post09 avoid_holding_currentown 
		 		avoid_holding_currentfound avoid_holding2_currentown avoid_holding2_currentfound /* max for dummies */
	 , by(ind_hhold);
	 #delimit cr
	
	foreach v of var * {
		label var `v' "`l`v''"
	}

	 compress
	 save $temp\big`yr'hhold.dta, replace
}
	

************************************************************************
*** PART E: INCORPORTE MEASURE OF DISCLOSED AND HIDDEN WEALTH ***
************************************************************************

 * Create measures of disclosed wealth
   do "$prog\makedata8_disclosedwealthCLEAN.do"

foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {  
 	
* Add disclosed wealth to big files
	use $temp\big`yr'hhold.dta, clear
	merge 1:1 ind_hhold using $data\hiddenwealth_vd.dta, keep(match master) nogen 
	cap drop total_wealth07*
	
* Add hidden wealth of evaders caught in audits (PIP, credit card)
	merge 1:1 ind_hhold using $data\hiddenwealth_pip.dta, keep(match master) nogen 
	cap drop total_wealth07*
	
* Generate total wealth
	quietly {
	gen total_wealthA = mwealth
	gen total_wealthB = mwealth
	gen total_wealthC = mwealth
	// voluntary disclosers 
		replace total_wealthA = mwealth if disclosertype==1 & year<vd_year_of_contact
		replace total_wealthA = mwealth + disclosed_wealth07A - da_wealth if disclosertype==2 & year<vd_year_of_contact
		replace total_wealthA = mwealth + disclosed_wealth07A if disclosertype==3 & year<vd_year_of_contact
		replace total_wealthB = mwealth - da_wealth + disclosed_wealth07B if (disclosertype==1 | disclosertype==2) & year<vd_year_of_contact
		replace total_wealthB = mwealth + disclosed_wealth07B if disclosertype==3 & year<vd_year_of_contact
		replace total_wealthC = mwealth - da_wealth + disclosed_wealth07C if (disclosertype==1 | disclosertype==2) & year<vd_year_of_contact
		replace total_wealthC = mwealth + disclosed_wealth07C if disclosertype==3 & year<vd_year_of_contact
	// caught evaders
		replace total_wealthA = mwealth if evadertype==1 & year<aud_year_start
		replace total_wealthA = mwealth + hidden_wealth07A - da_wealth if evadertype==2 & year<aud_year_start
		replace total_wealthA = mwealth + hidden_wealth07A if evadertype==3 & year<aud_year_start
		replace total_wealthB = mwealth - da_wealth + hidden_wealth07C if (evadertype==1 | evadertype==2) & year<aud_year_start
		replace total_wealthB = mwealth + hidden_wealth07C if evadertype==3 & year<aud_year_start
		replace total_wealthC = mwealth - da_wealth + hidden_wealth07C if (evadertype==1 | evadertype==2) & year<aud_year_start
		replace total_wealthC = mwealth + hidden_wealth07C if evadertype==3 & year<aud_year_start		
	}

* Final disclosed wealth measure = method C = 2007:
* largest pre-disclosure correction for decided cases
* dom wealth times mean offshore-domestic ratio for cases in process
	
*update May 18, 2017: also include the disclosed wealth here, and fix the missings
	gen x=1 if hidden_wealth07C==.
	replace hidden_wealth07C=0 if hidden_wealth07C==.
	gen y=1 if disclosed_wealth07C==.
	replace disclosed_wealth07C=0 if disclosed_wealth07C==.
	gen mwealth_pre = mwealth - hidden_wealth07C - disclosed_wealth07C
	
	gen vd_hidden_wealth = hidden_wealth07C + disclosed_wealth07C
	label variable vd_hidden_wealth "Disclosed wealth (method C)" 
	replace vd_hidden_wealth=. if x==1 & y==1
	replace disclosed_wealth07C=. if y==1
	replace hidden_wealth07C=. if x==1
	drop x y
	
	cap drop rank_w_pre
	cap drop rank_w
	drop mwealth
	rename total_wealthC mwealth
	
	cumul  mwealth_pre, gen(rank_w_pre)
	cumul  mwealth, gen(rank_w)
	
	compress 
	save $data\big`yr'hhold.dta, replace	 
}		
	
************************************************************************
*** PART F: CREATE SMALL FILES  AND INCLUDE STANDARD DEVIATIONS ***
************************************************************************	
*/
foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {

use $data\big`yr'hhold.dta, clear

* Create 127 wealth groups of wealth post-disclosure
	qui gen ptile_w=0
	qui replace ptile_w = trunc(100*rank_w) if rank_w<0.99
	qui replace ptile_w = min(trunc(1000*rank_w),999)/10 if rank_w>=0.99 & rank_w<0.999
	qui replace ptile_w = min(trunc(10000*rank_w),9999)/100 if rank_w>=0.999 & rank_w<0.9999
	qui replace ptile_w = min(trunc(100000*rank_w),99999)/1000 if rank_w>=0.9999
	qui replace ptile_w = round(ptile_w, .001) 
	qui drop rank_w

* Collapse
	foreach v of var _all {
		local l`v' : variable label `v'
		if `"`l`v''"' == "" local l`v' "`v'"
	}
	ds ptile_w, not 
	collapse (count) weight=ind_hhold (min) thwealth=mwealth (mean) `r(varlist)' (sd) disclosed_wealth07C_SD = disclosed_wealth07C, by(ptile_w)
 	foreach v of var * {
		label var `v' "`l`v''"
	}
	saveold $data\small`yr'hhold.dta, version(13) replace
}


************************************************************************
*** PART G: CREATE SMALL FILES BASED ON PRE-DISCLOSURE WEALTH AND INCLUDE STANDARD DEVIATIONS ***
************************************************************************	

foreach yr in 01 02 03 04 05 06 07 08 09 10 11 12 13 {

use $data\big`yr'hhold.dta, clear

* Create 127 wealth groups of wealth post-disclosure
	qui gen ptile_w_pre=0
	qui replace ptile_w_pre = trunc(100*rank_w_pre) if rank_w_pre<0.99
	qui replace ptile_w_pre = min(trunc(1000*rank_w_pre),999)/10 if rank_w_pre>=0.99 & rank_w_pre<0.999
	qui replace ptile_w_pre = min(trunc(10000*rank_w_pre),9999)/100 if rank_w_pre>=0.999 & rank_w_pre<0.9999
	qui replace ptile_w_pre = min(trunc(100000*rank_w_pre),99999)/1000 if rank_w_pre>=0.9999
	qui replace ptile_w_pre = round(ptile_w_pre, .001) 
	qui drop rank_w_pre

* Collapse
	foreach v of var _all {
		local l`v' : variable label `v'
		if `"`l`v''"' == "" local l`v' "`v'"
	}
	
	
	ds ptile_w_pre , not 
	collapse (count) weight=ind_hhold (min) thwealth=mwealth (mean) `r(varlist)'  (sd) disclosed_wealth07C_SD = disclosed_wealth07C, by(ptile_w_pre)
 	foreach v of var * {
		label var `v' "`l`v''"
	}
	saveold $data\small`yr'hhold_PRE.dta, version(13) replace

}

********


clear
cap log close

