clear
set more off

	global prog   "\\...\CLEAN\prog"
	global source "\\...\source_data\no"
	global data   "\\...\CLEAN\data"
    global temp   "\\...\data\no-backup"

cap log close
log using $prog\makedata4_ownershipCLEAN.log, replace

/*******************************************************************************
* Make direkt ownership shares, 2004-2013 for both persons and firms, from the 
* shareholder register and number of direct personal and firm owners in a firm, 
* 10 indirect layers, total 11 layers of ownership.
*
* We will not be able to attribute all firm variables to an ultimate individual 
* owner because:
*  a) we cut out the very small ownership shares when imputing hte indirect ownership
*  b) ownership shares are constructed based on observed ownership of direct 
*     shareholders. A considerable part of shares is held by institutions, public 
*     sector, foreigners.
*     make a fake personal ID==0 for ownership shares not attributable to indiviudal
*     must exclude this when attributing imputed retained profits.  
*******************************************************************************/

/* Find number of shares in the corporation, and calculate each owner's ownership 
   share, if being an individual or a corporation. Give the "unidentified" owners 
   foreigners or institutions) the ID=0, to find the unidentified share
   Must exclude ID=0 when attributing imputed retained profits. */

foreach yr in 04 05 06 07 08 09 10 11 12 13 {
*find total number of shares in the corporation held by external owners at yearend
use $source\ar_aksjonaer2.dta, clear
  keep aar ORGNR_NN ANTALL_3112 AKSJ_FNR_NN AKSJ_ORGNR_NN
  keep if aar==20`yr'
  rename aar year
  rename ORGNR_NN as_orgnr
  rename AKSJ_ORGNR_NN orgnr
  label var orgnr "firm as sharholder ID"
  label var as_orgnr "ID, firm in question"	
  rename ANTALL_3112 antall_3112
  rename AKSJ_FNR_NN ID
  drop if antall_3112==0
  drop if antall_3112<0
  drop if antall_3112==.
  drop if as_orgnr==.
  drop if as_orgnr==orgnr
*make a fake personal ID that aggregates all the ownership shares by those we cannot identify
  replace ID=0 if ID==. & orgnr==.
save $temp\temp_source_`yr', replace

*find total number of shares in the corporation
  collapse (sum) antall_3112, by(as_orgnr)
  rename antall_3112 total_shares
  sum total_shares, det
  drop if total_shares<1
  sort as_orgnr
save $temp\temp_total_shares, replace

* find each shareholder's (both individuals and corporations) number of shares 
*** idividual owners    
use $temp\temp_source_`yr', clear
  keep if ID!=.
  collapse (sum) antall_3112, by(ID as_orgnr)
  rename antall_3112 shares
  sum shares, det
  drop if shares<1
  sort as_orgnr
  merge n:1 as_orgnr using $temp\temp_total_shares
  keep if _m==3
  drop _merge
  gen ownshare=shares/total_shares
  label var ownshare "direct ownership share in firm"
  sum ownshare, det
 
*count number of individual direct owners in the firm at yearend
  sort as_orgnr
  gen x=1
  by as_orgnr: egen num_pers_owners=sum(x)
  label variable num_pers_owners "Number of unique personal direct owners in as_orgnr"
  drop x
  tab  num_pers_owners
  sum num_pers_owners, det
  sort ID as_orgnr
save  $data\direct_ownshares`yr'_ind, replace

*corporations' ownership shares 
use $temp\temp_source_`yr', clear
  keep if orgnr!=.
  sort orgnr as_orgnr 
  collapse (sum) antall_3112, by(orgnr as_orgnr)
  rename antall_3112 shares
  sum shares, det
  drop if shares<1
  sort as_orgnr
  merge n:1 as_orgnr using $temp\temp_total_shares
  keep if _m==3
  drop _merge
  gen ownshare=shares/total_shares
  label var ownshare "direct ownership share in firm"
  sum ownshare, det

*count number of corporate owners in the firm at yearend
  sort as_orgnr
  gen x=1
  by as_orgnr: egen num_corp_owners=sum(x)
  label variable num_corp_owners "Number of unique direct corporate owners in as_orgnr"
  drop x
  tab  num_corp_owners
  sum num_corp_owners, det
  sort orgnr as_orgnr
save  $data\direct_ownshares`yr'_firm, replace
}

/* Make indirect ownership shares through 10 layers of indirect ownership from 
   the shareholder register. To simplify the joinby routines and make the data 
   procedure workable, we only keep if the individual's ownshare is greater than 
   0.001%, ownshare>0.00001 */

foreach yr in 04 05 06 07 08 09 10 11 12 13 {
*** prepare auxiliary datasets ***
use $data\direct_ownshares`yr'_firm, clear
  keep as_orgnr ownshare orgnr
  rename orgnr owner_id
  rename as_orgnr mergeid
  rename ownshare shares_owner
  sort mergeid
save $temp\baseline_owner_firm, replace

use $data\direct_ownshares`yr'_ind, clear
  keep ID ownshare as_orgnr
  rename ID owner_id
  rename as_orgnr mergeid
  rename ownshare shares_owner
  sort mergeid
save $temp\baseline_owner_ind, replace

use $data\direct_ownshares`yr'_firm, clear
  keep as_orgnr ownshare orgnr
  rename as_orgnr owned_id
  rename orgnr mergeid
  sort mergeid
  rename ownshare shares_owned
save $temp\baseline_owned_firm, replace


*** construct layer 1: direct ownership shares of individuals
use $data\direct_ownshares`yr'_ind, clear
  keep ID as_orgnr ownshare 
  rename ID owner_id
  rename as_orgnr owned_id
  rename ownshare shares
save $temp\layer_1, replace
  sum shares, det


*** construct layer 2: indirect ownership shares with 1 intermediate corporation 
use $temp\baseline_owner_ind, clear
  count
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_2, replace
  sum shares, det

  
*** construct layer 3: indirect ownership shares with 2 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_3, replace
  sum shares, det

*** construct layer 4: indirect ownership shares with 3 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_4, replace
  sum shares, det

*** construct layer 5: indirect ownership shares with 4 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_5, replace
  sum shares, det
  
*** construct layer 6: indirect ownership shares with 5 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_6, replace
  sum shares, det  

*** construct layer 7: indirect ownership shares with 6 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_7, replace
  sum shares, det   

*** construct layer 8: indirect ownership shares with 7 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_8, replace
  sum shares, det 
  
*** construct layer 9: indirect ownership shares with 8 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_9, replace
  sum shares, det   
  
*** construct layer 10: indirect ownership shares with 9 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_10, replace
  sum shares, det   
  
*** construct layer 11: indirect ownership shares with 10 intermediate corporations 
  rename owned_id mergeid
  rename shares shares_owner
  keep if shares_owner>0.00001
  joinby mergeid using $temp\baseline_owned_firm
  gen shares = shares_owned * shares_owner
  drop mergeid shares_owned shares_owner
  collapse (sum) shares, by(owner_id owned_id)
save $temp\layer_11, replace
  sum shares, det   
  
/* Combine to get total ownership shares for all individual-corporation pairs */
use $temp\layer_1, clear
  forvalues x=2/11 {
  append using $temp\layer_`x'
}
  collapse (sum) shares, by(owner_id owned_id)
  sum shares, det 
***remember that ID==0 means unidentifable owner==institution&foreigner&funds&state
  sum shares if owner_id!=0, det
 
* make the data compatible with the rest of the data, change variable names back
  rename owner_id ID 
  rename owned_id as_orgnr 
  rename shares total_ownshare
  label var total_ownshare "total ownshare = direct + 11-layers-indirect"
  
* add on also the direct ownerhsip shares in the firm as a separate variable
  sort ID as_orgnr
  merge 1:1 ID as_orgnr using $data\direct_ownshares`yr'_ind
  keep ID as_orgnr total_ownshare ownshare
  replace ownshare=0 if ownshare==.

  sort as_orgnr ID
save $temp\temp_total_ownshares`yr'_ind, replace
  
preserve
* test that ownerhip shares does not exceed 1 
  collapse (sum) total_ownshare ownshare, by(as_orgnr)
  sum total_ownshare ownshare, det
restore

* how much are we loosing because of foreingers/unidentified domestics?
  drop if ID==0
  collapse (sum) total_ownshare ownshare, by(as_orgnr)
  sum total_ownshare ownshare, det
}


/* Add info on CEO and board members to the shareholding data and make variables 
   for CEO, board member, working in the firm. Also, add all individuals who are 
   board members and ceo's in the firm to the panel, in order to use it later, 
   even if they own no shares. */

foreach yr in 04 05 06 07 08 09 10 11 12 13 {
use $temp\temp_total_ownshares`yr'_ind, clear
  merge 1:1 as_orgnr ID using $data\board_`yr'
  replace total_ownshare=0 if _m==2
  drop _m
* distinguish btw board members and CEOs
  gen board=1 if CEO==0
  label var board "Board member"
  replace CEO=0 if CEO==.
  replace board=0 if board==.
*identify employees (made in Makedata3)
  merge 1:1 as_orgnr ID using $temp\lto`yr'
  drop if _m==2
  gen employee=(_m==3)
  drop _m ind_lto_wage
  tab employee, m
  keep  ID as_orgnr ownshare total_ownshare CEO board employee
  replace ownshare=0 if ownshare==.
  sort as_orgnr ID
  d
  sum
  compress
save $data\total_ownshares`yr'_ind.dta, replace
}

********
 
clear
log close
  
 
 
  
