* Computes wealth at market value in Sweden

* Part A: cleans SCB wealth data
* Part B: constructs market value wealth matching national accounts totals

clear
set more off


global source 	"\\...\SourceData" 
global progs 	"\\...\CLEAN\prog" 
global data		"\\...\CLEAN\data" 
global output	"\\...\CLEAN\data\output" 
global temp	    "\\...\temp"

* Ado file directory
    cap program drop gperc
	global ado_dir "$\\...\ProgramsNew\ado"
	sysdir set PERSONAL "$ado_dir"

cap log close
log using $progs\sw_makedata2_wealthincCLEAN_2017_07_25.log, replace


************************************************************************
*** PART A: CLEANS TAXABLE WEALTH DATA ***
************************************************************************

foreach yr in 99 00 01 02 03 04 05 06 07  {
    
	if `yr' <   50 local year = 20`yr' 
	if `yr' >=  50 local year = 19`yr'
	di "RUNNING YEAR `year'" 

	use $source\form`yr'_lev.dta, clear
	duplicates report lopnr
	sort lopnr fnettmv  
	by lopnr: keep if _n == _N
	sort lopnr
    sum
* Correct for discrepancies in variable names and coverage across years

	if `year' == 1999  {
		*missing variable for 1999, will be included below in "a_rest"
		qui gen fkuvpr = 0 
	}
	if `year' == 1999 | `year' == 2000 | `year' == 2001 {
		qui egen fofomv = rsum(fbfonmv  fakfmv)
		qui rename fkurfon frfonmv 
		*fkuutf already exists as 0 in 2001 
		qui cap drop fkuutf 
		qui gen fkuutf=0
	}
	if `year' == 1999 | `year' == 2000 | `year' == 2001 | `year' == 2002 | `year' == 2003 {
		qui rename fborm fbormny
	}
	if `year' == 2006 | `year' == 2007  {
		qui drop fkuvpr
		*only have declared value of bonds pre 2006, only fkuvpr
		qui rename fkuormv fkuvpr 
		
		
	}

* Create homogenous wealth variables and label
	qui rename lopnr ID
	label var ID "ID"
	
	qui rename fskulmv d_debt 
	label var d_debt "market-value reported debt"
	*Nb: variable for farming property, ffstjbm, includes farm houses, so farm houses, ffstejm, should not be included in total housing
	
	qui egen a_housing = rsum(ffstegm ffstego ffstfrm ffstjbm ffsthym ffsthyo ffstinm ftomtem ftomtfm fbormny)
	label var a_housing "market value reported housing and land property"
	
	qui gen a_f_housing = ffstum
	label var a_f_housing "market value reported foreign housing property"
	
	qui egen a_fix = rsum(fkubank fkuvpr fkuprem fopmv fovrmv)  
	label var a_fix "market-value reported bank deposits + bonds + options + convertibles"
	
	qui egen a_funds = rsum(frfonmv fofomv)  
	label var a_funds "market-value reported mutual funds (bond + equity)"
	
	qui egen a_shares = rsum(faktibmv  fotcmv  faalmv)
	*qui replace a_shares = a_shares + a_funds
	*label var a_shares "market value reported shares + funds"
	label var a_shares "market value reported shares"
	
	qui gen a_insurance  = ffors
	label var a_insurance "market value reported taxable capital insurances"
	
	qui gen a_f_insurance = fkuutf
	label var a_f_insurance "foreign insurances, market value 06-07"	 
	
	qui gen a_sum_financial_assets = ffinmv
	label var a_sum_financial_assets "market value reported financial assets"
	
	qui gen a_sum_real_assets = frealmv
	label var a_sum_real_assets "market value reported real assets"
	
	qui gen a_sum_total_assets = fsummv
	label var a_sum_total_assets "market value reported total assets"
	
	qui gen a_netwealth = fnettmv  
	label var a_netwealth "market value of reported net wealth"

* Correct discrepancy between total and components
	foreach var of varlist a_* d_* {
		qui replace `var' = 0 if `var' == .
	}
	qui gen a_rest = a_sum_total_assets - a_housing - a_f_housing - a_fix - a_funds - a_shares -  a_insurance - a_f_insurance
	* NB: a_rest large in 1999: we miss fovrtmv ("other assets") 
	* a_rest small after 1999 
	label var a_rest "correct for discrepancy between total assets and sum a_*"
	
	keep ID d_* a_*
	sort ID
	compress
	sum
	sum a_rest, det
	save $temp\wealth_`yr'.dta, replace
}

************************************************************************
*** PART B: MATCH TO MACRO TOTALS ***
************************************************************************
 
 foreach yr in 99 00 01 02 03 04 05 06 07  {

	if `yr' <   50 local year = 20`yr' 
	if `yr' >=  50 local year = 19`yr'
	di "RUNNING YEAR `year'" 

* Load macro parameters 
	qui insheet using "$data\parameters_swe.txt", clear names
	qui keep if year == `year'
	qui cap drop check
	foreach var of varlist _all {
		local `var'=`var'
	}

use $temp\wealth_`yr'.dta, clear

* Housing
	cap drop ma_housing
	qui egen ma_housing = rsum(a_housing a_f_housing)
	label variable ma_housing "Market value of housing (domestic + foreign)"
	qui su ma_housing, meanonly
	local scale_housing = `tthouse' / r(sum)
	di "SCALE UP FACTOR FOR HOUSING IN `year' = `scale_housing'"
	qui replace ma_housing = ma_housing * `scale_housing'
	qui su ma_housing, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE HOUSING IN YEAR `year': `total' BN SEK"
	assert `total' == round(`tthouse' / 1e9)

* Business assets: capitalize business income
	sort ID
	merge 1:1 ID using $data\income_`yr'.dta, keepusing(i_business) keep(match master) nogen
	qui replace i_business = 0 if i_business == .
	qui replace i_business = max(0, i_business)
	qui su i_business, meanonly 
	local cap_biz = `ttbiz' / r(sum)
	di "CAPITALIZATION FACTOR FOR BUSINESS ASSETS IN `year' = `cap_biz'"
	cap drop ma_business
	qui gen ma_business = i_business * `cap_biz'
	qui drop i_business
	label variable ma_business "Market-value of business & other non-financial assets"
	qui su ma_business, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE BUSINESS IN YEAR `year': `total' BN SEK"
	assert `total' == round(`ttbiz' / 1e9)

* Equities
	qui su a_shares, meanonly
	local toteq = r(sum)
	di "REPORTED EQUITY + FUND WEALTH IN `year' = `toteq' SEK = LISTED SHARES + MUTUAL FUNDS"
	// Micro-macro gap (= essentially unlisted shares): proportionally to dividends 
		*qui rename ID lopnr
		*merge 1:1 lopnr using $source\iot`yr'_lev.dta, keepusing(kkuutd) keep(match master) nogen
		merge 1:1 ID using $data\income_`yr'.dta, keepusing(kkuutd) keep(match master) nogen
		*qui rename lopnr ID
		qui replace kkuutd = 0 if kkuutd == . | kkuutd < 0
		qui su kkuutd, meanonly
		qui gen a_shares_imputed = (`ttequity' - `toteq') * kkuutd / r(sum)
		cap drop ma_equity
		qui gen ma_equity = a_shares + a_shares_imputed
		qui su ma_equity, meanonly
		local total = round(r(sum) / 1e9)
		di "TOTAL MARKET VALUE EQUITIES IN YEAR `year': `total' BN SEK"
		assert `total' == round(`ttequity' / 1e9)
	
* Fixed-income claims 
	cap drop ma_fix
	qui gen ma_fix = a_fix + a_rest
	label variable ma_fix "Market-value of fixed income claims"
	qui su ma_fix, meanonly
	local scale_fix = `ttfix' / r(sum)
	di "SCALE UP FACTOR FOR FIXED-INCOME CLAIMS IN `year' = `scale_fix'"
	qui replace ma_fix = ma_fix * `scale_fix'
	qui su ma_fix, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE FIXED INCOME CLAIMS IN YEAR `year': `total' BN SEK"
	assert `total' == round(`ttfix' / 1e9)

* Pension wealth: imputed as in Denmark and Norway
	merge 1:1 ID using $data\income_`yr'.dta, keepusing(i_wage i_pensions) keep(match master) nogen
	merge 1:1 ID using $data\perskar_`yr'.dta, keepusing(ind_age) keep(match master) nogen
	qui replace i_wage = 0 if i_wage == .
	qui replace i_pensions = 0 if i_pensions == .	
	cap drop ma_pension
	qui su i_wage if ind_age >= 20 & ind_age < 40, meanonly
	qui gen ma_pension = 0.1 * `ttpension' * i_wage / r(sum) if ind_age >= 20 & ind_age < 40 
	qui su i_wage if ind_age >= 40, meanonly
	qui replace ma_pension = 0.3 * `ttpension' * i_wage / r(sum) if ind_age >= 40
	qui replace ma_pension = 0 if ma_pension == .
	qui su i_pensions if ma_pension == 0 & i_pensions > 0, meanonly
	qui replace ma_pension = 0.6 * `ttpension' * i_pensions / r(sum) if ma_pension == 0 & i_pensions > 0
	qui su ma_pension, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE PENSION WEALTH IN YEAR `year': `total' BN SEK"
	assert `total' == round(`ttpension' / 1e9)	
	qui drop i_wage i_pensions ind_age
	// Add insurance assets
		qui replace a_insurance = 0 if a_insurance == .
		qui replace a_f_insurance = 0 if a_f_insurance == .
		qui replace ma_pension = ma_pension + a_insurance + a_f_insurance
		label variable ma_pension "Market value pension wealth + insurance"

* Debt (saved with minus sign)
	cap drop md_debt
	qui su d_debt, meanonly
	local scale_debt = `ttdebt' / r(sum)
	di "SCALE UP FACTOR FOR DEBT IN `year' = `scale_debt'"
	qui gen md_debt = - d_debt * `scale_debt'
	label variable md_debt "Market value debt (saved with minus sign)"
	qui su md_debt, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE DEBT IN YEAR `year': `total'BN  SEK"
	assert `total' == - round(`ttdebt' / 1e9)	

* Net wealth
	cap drop mwealth
	qui egen mwealth = rsum(ma_housing ma_fix ma_equity ma_business ma_pension md_debt)
	label variable mwealth "Net wealth at market value"
	qui su mwealth, meanonly
	local total = round(r(sum) / 1e9)
	di "TOTAL MARKET VALUE NET WEALTH IN YEAR `year': `total' BN SEK"


* Save
	qui compress
	sort ID
	d, det
	sum
	save $data\wealth_`yr'.dta, replace
}

* Check: wealth distribution household-level
	mat drop _all
	foreach yr in 99 00 01 02 03 04 05 06 07  {
		use $data\wealth_`yr'.dta, clear

		merge 1:1 ID using $data\perskar_`yr'.dta, keepusing(ind_hhold) keep(match master) nogen
		collapse (sum) mwealth* ma_* md_*, by(ind_hhold)
		qui de mwealth 
		local nbhhold = r(N)
		qui gen net_housing = ma_housing + md_debt
		shcomp mwealth net_housing ma_fix ma_equity ma_business ma_pension, matname(mwealth`yr')
			mat mwealth`yr'  = (`yr', `nbhhold', mwealth`yr')
			mat mwealth   = (nullmat(mwealth)  \ mwealth`yr')
	}
	di "TOP WEALTH SHARE AND THEIR COMPOSITION (MWEALTH)"
	mat list mwealth 
	clear
	svmat mwealth, names(col)
	qui compress
	export excel using "$output\distrib_mwealth_hhold.xlsx", first(var) replace 

* Wealth by g-percentile household-level
mat drop _all
	foreach yr in 99 00 01 02 03 04 05 06 07  {

		use $data\wealth_`yr'.dta, clear
		
		merge 1:1 ID using $data\perskar_`yr'.dta, keepusing(ind_hhold) keep(match master) nogen
		merge 1:1 ID using $data\income_`yr'.dta, keepusing(i_inc i_inckg tax_total) keep(match master) nogen
		
		collapse (sum) mwealth* ma_* md_* i_inc* tax_total, by(ind_hhold)
		
		qui gen net_housing = ma_housing + md_debt
		gperc mwealth net_housing ma_fix ma_equity ma_business ma_pension i_inc i_inckg tax_total, matname(mwealth`yr')
		clear
		svmat mwealth`yr', names(col)
		qui compress
		export excel using "$output\gperc_mwealth_inc`yr'_hholdCLEAN.xlsx", first(var) replace 
	}

	*******
log close
clear  
 

	
